;-----------------------------------------------------------------------
;
;       int DualSlope(byte signal_source)
;
;	steuerroutine fuer den dualslope wandler
;
;	signalsource = 0 -> AutoZero messung (Messeingang wird auf 0 gelegt)
;	signalsource = 1 -> Signal   messung (Messeingang wird auf )
;
;	Timer 0 dient als Zaehler
;
;
;-----------------------------------------------------------------------

?PR?DUALSLOPE?	segment CODE

		CSEG at 0bh      	;vector fuer interrupt service routine
		ljmp	DualServ

RSEG		?PR?DUALSLOPE?

INTTIME		equ	49152+13	;2^16 - 2^14 * 12 / fclk ca. 24.57 mS Integration time
					;@ 11.0592Mhz Clock (+13 cycles fuer delay Interrupt
					;nach Mux umschaltung )

DELAY_1		equ	12		;6.144 mS @ 11.0592 Mhz Takt
		public	_DualSlope


_DualSlope:	setb	P1.2		;ADC auf Standby stellen

		mov	r1,#DELAY_1	;Standby ruhe pause
StByWait_2:	mov	r0,#0
StByWait_1:	djnz	r0,StByWait_1	;6144 Takte
		djnz	r1,StByWait_2

		clr	TR0		;timer 0 stop
		mov	a,TMOD		;setze timer 0 auf 16 bit timer mode
		anl	a,#11110000b
		orl	a,#00000001b
		mov	TMOD,a
					;lade integration time
		mov	TH0,#high (INTTIME)
		mov	TL0,#low  (INTTIME)
		clr	TF0		;clear ueberlauf flag
		setb	ET0		;freigabe interrupt

		mov	a,r7		;Zero messung oder Signal messung
		jz	AutozeroMess

		mov	a,P1
		anl	a,#11100011b
		orl	a,#00001000b	;MUX auf Signal Integrate stellen
		mov	P1,a
		sjmp	StartMess	

AutozeroMess:	mov	a,P1
		anl	a,#11100011b
		orl	a,#00011000b
		mov	P1,a		;MUX auf Autozero stellen

		nop			
		nop			;zeit ausgleich gegenueber sjmp StartMess

StartMess:	setb	TR0		;start timer 0

WaitForInt:	jb	ET0,WaitForInt	;warte bis Interrupt aufgetreten ist
					;dann fertig mit Integrate
					;ET0 wird von der Interrupt service routine
					;geloescht , dann wird von der Serviceroutine
					;die Deintegrate Phase eingeleitet

WaitDeInt:	jb	TF0,OverFlow	;timer ueberlauf ?
		jb	INT0,WaitDeInt	;warte bis fertig DeIntegriert ist
OverFlow:	clr	TR0		;stop timer
		setb	P1.2		;AD-Converter auf Standby stellen
		mov	r7,TL0		;timer auslesen
		mov	r6,TH0
		ret


;------------------------------------------------------------------------
;
;	Interrupt service routine fuer DualSlope AD- Converter
;	stops Intergratephase und loescht das ET0 flag
;	start DeIntegratePhase
;	Clear Timer 0
;
;------------------------------------------------------------------------

DualServ:      	push	acc             ;2 cyc
		push	psw             ;2 cyc
		clr	ET0		;1 cyc Interrupt von Timer 0 sperren
		clr	TR0		;1 cyc stop timer
		mov	TL0,#0          ;2 cyc
		mov	TH0,#0		;2 cyc Timer 0 loeschen

		orl	TMOD,#00001000b	;2 cyc Timer 0 auf externe freigabe umschalten

;;		mov	a,P3            ;1 cyc
;;		setb	acc.4		;1 cyc P3.4 = 1
;;		setb	acc.2		;1 cyc P3.2 = 1
;;		clr	acc.3           ;1 cyc P3.3 = 0
;;		mov	P3,a            ;1 cyc

;;;	setb	P3.4            ;1 cyc
;;;	clr	P3.3            ;1 cyc

		clr	P1.3		;1 cyc start Deintegrate


		setb	TR0		;timer freigeben (DeIntegrate time)
		pop	psw
		pop	acc
		reti

		end
